(function () {
    // Client side property renderer for NCD. Request page html from the delivery site and partial refresh the property DOM.

    // property name with these prefix need to be removed when searching DOM
    var propertyPrefix = ["icontent_", "ichangetrackable_", "iversionable_", "iroutable_"];

    function selector(propertyName) {
        propertyPrefix.forEach(function (prefix) {
            if (propertyName.indexOf(prefix) == 0) {
                propertyName = propertyName.substring(prefix.length);
            }
        });

        return "[data-epi-property-name='" + propertyName + "' i], [data-epi-edit='" + propertyName + "' i]";
    }

    function updateHtmlDom(newHtmlContent, updatedProperties) {
        var page = document.createElement("html");
        page.innerHTML = newHtmlContent;

        updatedProperties.forEach(function (property) {
            if (!property.successful) {
                return;
            }

            var updatedNode = page.querySelector(selector(property.name));
            document.querySelectorAll(selector(property.name)).forEach(function (node) {
                // skip rendering properties that set `data-epi-edit` or has `data-epi-property-render`=="none"
                if (node.getAttribute("data-epi-edit")
                    || node.getAttribute("data-epi-property-render") == "none") {
                    return;
                }
                if (updatedNode) {
                    node.innerHTML = updatedNode.innerHTML;
                } else {
                    node.remove();
                }
            });
        });
    }

    function getUpdatedPageContentAsync(contentLink) {
        // Replace url with updated ContentLink
        var url = location.href.replace(/,,[\d_]+/, ",," + contentLink);
        return fetch(url, {cache: "no-cache"}).then(function (response) {
            if (response.ok) {
                return response.text();
            } else {
                throw new Error(response.statusText);
            }
        });
    }

    function subscribeEvent() {
        if (window.epi.isEditable) {
            window.epi.subscribe("contentSaved", function (message) {
                if (!message.properties || !Array.isArray(message.properties)) {
                    return;
                }

                getUpdatedPageContentAsync(message.contentLink).then(function (data) {
                    updateHtmlDom(data, message.properties);
                }).catch(function (error) {
                    console.error("An error has occurred during fetch operation.", error);
                });
            });
        }
    }

    window.addEventListener("load", function () {
        if (!window.epi) {
            return;
        }
        if (window.epi.ready === true) {
            subscribeEvent();
        } else {
            window.epi.subscribe("epiReady", subscribeEvent);
        }
    });

})();
